//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "UwMonitor.h"
#include <registry.hpp>
#include "UdlgAboutMonitor.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "Trayicon"
#pragma resource "*.dfm"
TwMonitor *wMonitor;
//---------------------------------------------------------------------------

//----- Constructor ------------------------------------------------------------
// how: 1) Set some defaults
//      2) load some data from the registry
//      3) create a new MonitorThread and start it
//------------------------------------------------------------------------------
__fastcall TwMonitor::TwMonitor(TComponent* Owner)
   : TForm(Owner){
   cbLevel->ItemIndex = 0;                      //1
   FRootKey = "Software\\Gattung\\Monitor";     //2
   TRegistry *reg = new TRegistry;
   try {
      reg->OpenKey(FRootKey,false);
      if ( reg->ReadBool("StayOnTop") )
         FormStyle = fsStayOnTop;
      else
         FormStyle = fsNormal;

      Height = reg->ReadInteger("Height");
      Top = reg->ReadInteger("Top");
      Left = reg->ReadInteger("Left");
      Width = reg->ReadInteger("Width");
      cbLevel->ItemIndex = reg->ReadInteger("Level");

   } catch ( ... ){
   }
   delete reg;
   monitorThread = new TMonitorThread(true);   //3
   monitorThread->OnMessage = &onMessage;
   monitorThread->Resume();

}//-----------------------------------------------------------------------------

//----- Destructor -------------------------------------------------------------
// how: 1) tells the thread to terminateand deletes it;
//      2) Save information to the registry
//------------------------------------------------------------------------------
__fastcall TwMonitor::~TwMonitor(){
   monitorThread->Terminate();            //1
   monitorThread->WaitFor();
   delete monitorThread;
   TRegistry *reg = new TRegistry;
   try {
      reg->OpenKey(FRootKey,true);
      reg->WriteBool("StayOnTop", FormStyle == fsStayOnTop);
      reg->WriteInteger("Height", Height);
      reg->WriteInteger("Top", Top);
      reg->WriteInteger("Left", Left);
      reg->WriteInteger("Width", Width);
      reg->WriteInteger("Level", cbLevel->ItemIndex);
   }__finally {
      delete reg;
   }
}//-----------------------------------------------------------------------------

//----- onMessage --------------------------------------------------------------
//Purpose: is invoced by the MonitorThread to display information
//     in: Sender = firing thread
//         Type = type (error, warning, info) of the received message
//         Message = text of the message
//         ErrCode = code, that migh contain further information
//    how: 1) the fontcolor is set depending on the type of message received
//         2) if the errorcode is not zero
//         3)        message and errorcode are inserted at the top of the rtf window
//         4) else only the message is inserted at the top of the rtf window
//         5) the beginning of the message is set as hint to the tray symbol
//------------------------------------------------------------------------------
void __fastcall TwMonitor::onMessage(TThread* Sender, TMonintorMessageType Type,
   String Message, int ErrCode){
   wMonitor->cMessage->SelStart = 0;                                 //1
   switch ( Type ){
      case mmt_info:
         if ( wMonitor->cbLevel->ItemIndex > 0 )
            return;
         wMonitor->cMessage->SelAttributes->Color = clLime;
         break;
      case mmt_warning:
         if ( wMonitor->cbLevel->ItemIndex > 1 )
            return;
         wMonitor->cMessage->SelAttributes->Color = clYellow;
         break;
      case mmt_error:
         wMonitor->cMessage->SelAttributes->Color = clRed;
         break;
      default:
         wMonitor->cMessage->SelAttributes->Color = clWhite;
   }//swich
   String t;
   if ( ErrCode != 0 )                                                  //2
      t = Message + " ("+String(ErrCode)+")";                           //3
   else
      t= Message;                                                       //4
   wMonitor->cMessage->Lines->Insert(0,t);                              //3,4
   wMonitor->cTrayIcon->Hint = ("Monitor: "+t).SubString(1,40);         //5
}//-----------------------------------------------------------------------------

//----- btnSave ----------------------------------------------------------------
//Purpose: asks for a filename and save the file as rtf-document
//------------------------------------------------------------------------------
void __fastcall TwMonitor::btnSaveClick(TObject *Sender){
   if ( dlgSave->Execute() ){
      cMessage->Lines->SaveToFile(dlgSave->FileName);
   }
}//-----------------------------------------------------------------------------

//----- btnClear ---------------------------------------------------------------
//Purpose: Clears the rtfMemo
//Attention: All Information will be lost!
//------------------------------------------------------------------------------
void __fastcall TwMonitor::btnClearClick(TObject *Sender){
   cMessage->Lines->Clear();
}//-----------------------------------------------------------------------------

//----- btnStayOnTop -----------------------------------------------------------
//Purpose: toggles the Formstyle between fsNormal and fsStayOnTop
void __fastcall TwMonitor::btnStayOnTopClick(TObject *Sender){
   if ( FormStyle == fsStayOnTop )
      FormStyle = fsNormal;
   else
      FormStyle = fsStayOnTop;
   btnStayOnTop->Down = ( FormStyle == fsStayOnTop );
}//---------------------------------------------------------------------------


//----- btnAbout ---------------------------------------------------------------
// Purpose: Displays the about dialog
//------------------------------------------------------------------------------
void __fastcall TwMonitor::btnAboutClick(TObject *){
   dlgAboutMonitor->ShowModal();
}//-----------------------------------------------------------------------------

//----- btnHelp ----------------------------------------------------------------
//Purpose: opens the index.html file wich contains more information
//------------------------------------------------------------------------------
void __fastcall TwMonitor::btnHelpClick(TObject *Sender){
   ShellExecute(NULL, "open", "index.html", NULL, NULL, SW_SHOWNORMAL);
}//-----------------------------------------------------------------------------

